@@ -1,7 +1,6 @@
 import logging
 
 from django.db.models.query import RawQuerySet
-
 from isc_common.http.DSRequest import DSRequest
 from isc_common.models.audit import AuditManager, AuditQuerySet
 
@@ -9,7 +8,23 @@ logger = logging.getLogger(__name__)
 
 
 class TreeAuditModelQuerySet(AuditQuerySet):
-    def get_descendants(self, id=None, start=None, end=None, child_id='child_id', parent_id='parent_id', child_parent_value=None, include_self=True, where_clause='', order_by_clause='', distinct='', fields='*') -> RawQuerySet:
+    def delete(self):
+        raise Exception('Удаление без проверки !!!!')
+        return super().delete()
+
+    def get_descendants(self,
+                        id=None,
+                        start=None,
+                        end=None,
+                        child_id='child_id',
+                        parent_id='parent_id',
+                        child_parent_value=None,
+                        include_self=True,
+                        where_clause='',
+                        order_by_clause='',
+                        distinct='',
+                        sql_text=None,
+                        fields='*') -> RawQuerySet:
         db_name = self.model._meta.db_table
 
         if id != None:
@@ -20,7 +35,9 @@ class TreeAuditModelQuerySet(AuditQuerySet):
             elif not isinstance(id, tuple):
                 raise Exception(f'id must be list or int')
 
-        if id != None:
+        if sql_text != None:
+            res = super().raw(sql_text)
+        elif id != None:
             sql_text = f'''WITH RECURSIVE r AS (
                                 SELECT *, 1 AS level
                                 FROM {db_name}
@@ -36,7 +53,7 @@ class TreeAuditModelQuerySet(AuditQuerySet):
     
                             select {distinct} {fields} from r {where_clause} {order_by_clause} limit %s offset %s
                         '''
-            logger.debug(f'sql: {sql_text}')
+            # logger.debug(f'sql: {sql_text}')
             res = super().raw(sql_text, params=((id,), end, start))
         else:
             child_parent_value_str = f' = {child_parent_value}'
@@ -55,7 +72,7 @@ class TreeAuditModelQuerySet(AuditQuerySet):
 
                                         select {fields} from r {where_clause} {order_by_clause} limit %s offset %s
                                     '''
-            logger.debug(f'sql: {sql_text}')
+            # logger.debug(f'sql: {sql_text}')
             res = super().raw(sql_text, params=(end, start))
 
         return res
@@ -88,7 +105,18 @@ class TreeAuditModelQuerySet(AuditQuerySet):
 
         return res
 
-    def get_descendants_count(self, id='id', limit=None, offset=None, child_id='child_id', parent_id='parent_id', child_parent_value=None, include_self=True, where_clause='', distinct='') -> int:
+    def get_descendants_count(self,
+                              id='id',
+                              limit=None,
+                              offset=None,
+                              child_id='child_id',
+                              parent_id='parent_id',
+                              child_parent_value=None,
+                              include_self=True,
+                              where_clause='',
+                              distinct='',
+                              sql_text=None
+                              ) -> int:
         return len(self.get_descendants(
             id=id,
             start=offset,
@@ -98,7 +126,8 @@ class TreeAuditModelQuerySet(AuditQuerySet):
             child_parent_value=child_parent_value,
             include_self=include_self,
             where_clause=where_clause,
-            distinct=distinct
+            distinct=distinct,
+            sql_text=sql_text
         ))
 
 
@@ -110,7 +139,20 @@ class TreeAuditModelManager(AuditManager):
         res = self.get_queryset().get_range_rows(start=request.startRow, end=request.endRow, function=function, distinct_field_names=distinct_field_names, json=request.json)
         return res
 
-    def get_descendants(self, id=None, limit=None, offset=None, child_id='child_id', parent_id='parent_id', child_parent_value=None, include_self=True, where_clause='', order_by_clause='', distinct='', fields='*') -> RawQuerySet:
+    def get_descendants(self,
+                        id=None,
+                        limit=None,
+                        offset=None,
+                        child_id='child_id',
+                        parent_id='parent_id',
+                        child_parent_value=None,
+                        include_self=True,
+                        where_clause='',
+                        order_by_clause='',
+                        distinct='',
+                        fields='*',
+                        sql_text=None,
+                        ) -> RawQuerySet:
         return self.get_queryset().get_descendants(
             id=id,
             start=offset,
@@ -122,10 +164,21 @@ class TreeAuditModelManager(AuditManager):
             where_clause=where_clause,
             order_by_clause=order_by_clause,
             distinct=distinct,
-            fields=fields
+            fields=fields, sql_text=sql_text
         )
 
-    def get_parents(self, id=None, limit=None, offset=None, child_id='child_id', parent_id='parent_id', include_self=True, where_clause='', order_by_clause='', fields='*') -> RawQuerySet:
+    def get_parents(self,
+                    id=None,
+                    limit=None,
+                    offset=None,
+                    child_id='child_id',
+                    parent_id='parent_id',
+                    include_self=True,
+                    where_clause='',
+                    order_by_clause='',
+                    fields='*',
+                    sql_text=None,
+                    ) -> RawQuerySet:
         return self.get_queryset().get_parents(
             id=id,
             start=offset,
@@ -135,10 +188,22 @@ class TreeAuditModelManager(AuditManager):
             include_self=include_self,
             where_clause=where_clause,
             order_by_clause=order_by_clause,
-            fields=fields
+            fields=fields,
+            sql_text=sql_text
         )
 
-    def get_descendants_count(self, id=None, limit=None, offset=None, child_id='child_id', parent_id='parent_id', child_parent_value=None, include_self=True, where_clause='', distinct='') -> int:
+    def get_descendants_count(self,
+                              id=None,
+                              limit=None,
+                              offset=None,
+                              child_id='child_id',
+                              parent_id='parent_id',
+                              child_parent_value=None,
+                              include_self=True,
+                              where_clause='',
+                              distinct='',
+                              sql_text=None,
+                              ) -> int:
         return self.get_queryset().get_descendants_count(
             id=id,
             limit=limit,
@@ -148,7 +213,8 @@ class TreeAuditModelManager(AuditManager):
             child_parent_value=child_parent_value,
             include_self=include_self,
             distinct=distinct,
-            where_clause=where_clause
+            where_clause=where_clause,
+            sql_text=sql_text
         )
 
     def get_queryset(self):
